unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sqlite3conn, sqldb, db, IniFiles, FileUtil,
  Forms, Controls, Graphics, Dialogs, DBGrids, PairSplitter,
  DbCtrls, Menus, StdCtrls, StringHelpers;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Datasource: TDatasource;
    DBGrid1: TDBGrid;
    DBNavigator1: TDBNavigator;
    Connection: TSQLite3Connection;
    Query: TSQLQuery;
    Transaction: TSQLTransaction;
    procedure Button1Click(Sender: TObject);
    procedure FormClose(Sender: TObject;
      var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure QueryAfterOpen(DataSet: TDataSet);
  private
    ProjectIniFile: String;
    procedure FillSomeData;
    procedure LoadIni;
    procedure SaveIni;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  try
    ProjectIniFile := Application.Location + PathDelim + 'project.ini';
    Connection.DatabaseName := Application.Location + PathDelim + 'test.sql';
    Connection.Connected:=True;

    Query.SQL.Text:=
      'CREATE TABLE IF NOT EXISTS foo ('+
      'id INTEGER PRIMARY KEY AUTOINCREMENT, '+
      'str1 VARCHAR(100), '+
      'str2 VARCHAR(100), '+
      'str3 VARCHAR(100), '+
      'str4 VARCHAR(100));';
    Query.ExecSQL;
    Transaction.Commit;

    Query.SQL.Text:='SELECT * FROM foo;';
    Query.PacketRecords := 100;                         // mindestens 100 Records laden, wenn vorhanden
    Query.Open;

    if Query.RecordCount < 10 then
      FillSomeData;
  except
  end;
end;

procedure TForm1.QueryAfterOpen(DataSet: TDataSet);
begin
  LoadIni;
end;

procedure TForm1.FillSomeData;
var
  i: Integer;
begin
  for i := 0 to 99 do
  begin
    Query.Append;
    Query.Fields.FieldByName('str1').AsString := RandomString;
    Query.Fields.FieldByName('str2').AsString := RandomString;
    Query.Fields.FieldByName('str3').AsString := RandomString;
    Query.Fields.FieldByName('str4').AsString := RandomString;
    Query.Post;
  end;
end;

procedure TForm1.LoadIni;
var
  Ini: TIniFile;
  List: TList;
  i, i2: Integer;
  AFieldName: String;
begin
  if not Query.Active then Exit;

  List := TList.Create;
  Ini := TIniFile.Create(ProjectIniFile);
  try
    for i := 0 to DBGrid1.Columns.Count - 1 do
      List.Add(DBGrid1.Columns.Items[i]);

    for i := 0 to DBGrid1.Columns.Count - 1 do
    begin
      AFieldName := Ini.ReadString('Grid', 'Column' + IntToStr(i), DBGrid1.Columns.Items[i].FieldName);
      for i2 := 0 to List.Count - 1 do
        if AFieldName = TColumn(List[i2]).FieldName then
        begin
          TColumn(List[i2]).Index := i;
          Break;
        end;
    end;
  except
    on e: Exception do
      ShowMessage('TForm1.LoadIni: ' + e.Message);
  end;
  Ini.Free;
  List.Free;
end;

procedure TForm1.SaveIni;
var
  Ini: TIniFile;
  i: Integer;
begin
  if not Query.Active then Exit;

  Ini := TIniFile.Create(ProjectIniFile);
  try
    for i := 0 to DBGrid1.Columns.Count - 1 do
      Ini.WriteString('Grid', 'Column' + IntToStr(i), DBGrid1.Columns.Items[i].FieldName);
  except
    on e: Exception do
      ShowMessage('TForm1.SaveIni: ' + e.Message);
  end;
  Ini.Free;
end;

procedure TForm1.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
  SaveIni;
  Query.ApplyUpdates;
  Transaction.Commit;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i: Integer;
  List: TList;
begin
  List := TList.Create;
  for i := 0 to DBGrid1.Columns.Count - 1 do
    List.Add(DBGrid1.Columns.Items[i]);
  for i := 0 to List.Count - 1 do
    TColumn(List[i]).Index := List.Count - i - 1;
  List.Free;
end;

end.

